// Libraries
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <TTBOUNCE.h>
#include <EEPROM.h>
extern "C" {
#include "user_interface.h"
}

//======SERVER PART======================================================
const char* ssid     = "SSID";
const char* password = "Passwort";
ESP8266WebServer server ( 80 );
char htmlResponse[3000];
//======LOKALER PART======================================================
os_timer_t myTimer;
bool tickOccured;
bool timerrunning;
int RelaisPin = 4;                 // GPIO13 Blaue LED auf dem Sonoff
TTBOUNCE b = TTBOUNCE(5);

int time_ss = 6000;
int time_ds = 12000;

void handleRoot() {

  snprintf ( htmlResponse, 3000,
             "<!DOCTYPE html>\
<html lang=\"en\">\
  <head>\
    <meta charset=\"utf-8\">\
    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">\
  </head>\
  <body>\
          <h1>Zeiten für Singleshot und Doubleshot</h1>\
          <h3>Singleshot: %d% ms </h3>\
          <input type='text' name='date_ss' id='date_ss' size=2 autofocus> ms \
          <h3>Doubleshot: %d% ms</h3>\
          <input type='text' name='date_ds' id='date_ds' size=2 autofocus> ms \
          <div>\
          <br><button id=\"save_button\">Save</button>\
          </div>\
    <script src=\"https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js\"></script>\
    <script>\
      var ds;\
      var ss;\
        $('#save_button').click(function(e){\
        e.preventDefault();\
        ss = $('#date_ss').val();\
        ds = $('#date_ds').val();\
        $.get('/save?ss=' + ss + '&ds=' + ds, function(data){\
          console.log(data);\
        });\
        });\
    </script>\
  </body>\
</html>",
             time_ss,
             time_ds
           );
  server.send ( 200, "text/html", htmlResponse );
  //location.reload();\
}


void handleSave() {
  if (server.arg("ss") != "") {
    Serial.println("Singleshot: " + server.arg("ss"));
    time_ss = server.arg("ss").toInt();
    eeWriteInt(0, server.arg("ss").toInt());
  }
  if (server.arg("ds") != "") {
    Serial.println("Doubleshot: " + server.arg("ds"));
    time_ds = server.arg("ds").toInt();
    eeWriteInt(4, server.arg("ds").toInt());
  }

}

// start of timerCallback
void timerCallback(void *pArg) {
  Serial.println("Tick OccurredCB");
  tickOccured = true;
}

void setup() {
  //======SERVER PART======================================================
  // Start serial
  Serial.begin(115200);
  delay(300);
  WiFi.mode(WIFI_STA);

  // Connecting to a WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);
  
  int wifi_retry = 0;
  while (WiFi.status() != WL_CONNECTED && wifi_retry < 20) {
    wifi_retry++;
    delay(500); 
    Serial.print(" . ");
  }

  if(wifi_retry <= 19) {
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
  }
  else{
    Serial.println();
    Serial.println("connect failed");
    digitalWrite(RelaisPin, HIGH);  // turn LED ON
    Serial.println("engine toggled");
    if (WiFi.status() != WL_CONNECTED ){
      delay(100); 
    }
    digitalWrite(RelaisPin, LOW);
    Serial.println("engine off");
  }

  server.on ( "/", handleRoot );
  server.on ("/save", handleSave);

  server.begin();
  Serial.println ( "HTTP server started" );
  //======LOKALER PART======================================================
  os_timer_setfn(&myTimer, timerCallback, NULL);
  pinMode(RelaisPin, OUTPUT);       // GPIO13 als Ausgang definieren
  b.attachClick(click);         //attach the click method to the click event
  b.attachDoubleClick(doubleClick);//attach the double click method to the double click event
  b.attachPress(press);        //attach the press method to the press event
  delay(50);
  digitalWrite(RelaisPin, LOW);  // turn LED ON
  tickOccured = false;
  timerrunning = false;
  EEPROM.begin(8);  //Initialize EEPROM
  time_ss = eeGetInt(0);
  time_ds = eeGetInt(4);
}

void loop() {
  //======SERVER PART======================================================
  server.handleClient();
  //======LOKALER PART======================================================
  b.update();
  delay(10);

  if (tickOccured == true)
  {
    Serial.println("Timer abgelaufen");
    tickOccured = false;
    digitalWrite(RelaisPin, LOW);
    os_timer_disarm(&myTimer);
    timerrunning = false;
    
    int wifi_retry = 0;
    while(WiFi.status() != WL_CONNECTED && wifi_retry < 5 ) {
      wifi_retry++;
      Serial.println("WiFi not connected. Try to reconnect");
      WiFi.disconnect();
      WiFi.mode(WIFI_OFF);
      WiFi.mode(WIFI_STA);
      WiFi.begin(ssid, password);
      delay(200);
  }}

  yield();  // or delay(0);
  
  

}


void click() {
  if (!timerrunning)
  {
    Serial.println("Clicked");
    digitalWrite(RelaisPin, HIGH);  // turn LED ON
    os_timer_arm(&myTimer, time_ss, false);
    timerrunning = true;
  }
  else
  {
    Serial.println("ABORT!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
    tickOccured = false;
    digitalWrite(RelaisPin, LOW);
    os_timer_disarm(&myTimer);
    timerrunning = false;
  }
}

void doubleClick() {
  if (!timerrunning)
  {
    Serial.println("DoubleClicked");
    digitalWrite(RelaisPin, HIGH);  // turn LED ON
    os_timer_arm(&myTimer, time_ds, false);
    timerrunning = true;
  }
  else
  {
    Serial.println("ABORT!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
    tickOccured = false;
    digitalWrite(RelaisPin, LOW);
    os_timer_disarm(&myTimer);
    timerrunning = false;
  }
}

void press() {
  if (!timerrunning)
  {
    Serial.println("LongClicked");
    digitalWrite(RelaisPin, HIGH);  // turn LED ON
    os_timer_arm(&myTimer, 1000, false);
    timerrunning = true;
  }
  else
  {
    Serial.println("ABORT!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
    tickOccured = false;
    digitalWrite(RelaisPin, LOW);
    os_timer_disarm(&myTimer);
    timerrunning = false;
  }
}

void eeWriteInt(int pos, int val) {
  byte* p = (byte*) &val;
  EEPROM.write(pos, *p);
  EEPROM.write(pos + 1, *(p + 1));
  EEPROM.write(pos + 2, *(p + 2));
  EEPROM.write(pos + 3, *(p + 3));
  EEPROM.commit();
}

int eeGetInt(int pos) {
  int val;
  byte* p = (byte*) &val;
  *p        = EEPROM.read(pos);
  *(p + 1)  = EEPROM.read(pos + 1);
  *(p + 2)  = EEPROM.read(pos + 2);
  *(p + 3)  = EEPROM.read(pos + 3);
  return val;
}
